import java.awt.*;

/**
 * Extends Tile class. Represents a normal floor tile.
 * @author Nakul Dureja
 * @version 1.0 
 *
 */
public class Floor extends Tile{

	Item item;
	boolean isItem;

	/**
	 * Parameterized Constructor for the Floor class : Item on the Floor
	 * @param item Item object
	 */
	public Floor(Item item)
	{
		this.item=item;
		isItem = true;
	}
	
	
	/**
	 * Constructor for Floor class without parameter :  Normal Floor
	 */
	public Floor()
	{
		this.item=null;
		isItem = false;
	}

	
	/**
	 * Overridden abstract method. Floor is passable
	 * @return boolean true
	 */
	public boolean isPassable() {
		return true;
	}


	/**
	 * Overidden drawTile method. Draws the floor.
	 * @param g Graphics object
	 * @param x x coordinate
	 * @param y y coordinate
	 */
	public void drawTile(Graphics g, int x, int y) {
		
		// IF there is no item on the floor paint blank tile
		if(!this.isItem)
		{
			g.setColor(Color.BLACK);
			g.fillRect(x, y, width, height);

		}
		else // Draw the item
		{
			g.setColor(Color.BLACK);
			g.fillRect(x, y, width, height);
			item.drawItem(g,  x, y);
		}
	}
	

}