import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.Timer;
/**
 * This class contians the main method. It is the "gate-keeper" of the game.
 * It contains references to other subclasses which allows the player to interact
 * with the game. 
 * 
 * @author NAKUL DUREJA
 * @version 2.0
 *
 */
public class Game {

	// Static Variables 
	static LevelPanel levelPanel;

	//Instance Variables : Protected
	protected Level level;
	protected Player player;
	protected ArrayList<Monster> monsters;
	protected JLabel score;
	protected JLabel status;
	protected char[][] rep1;
	protected char[][] rep2;
	protected Timer t1; // Slow Timer for monster update
	protected Timer t2; // Fast Timer for player stats and collision  detection

	/**
	 * Win method: Checks to see if the level is finished or the player has won
	 */
	public void win()
	{
		// Player has collected all the gems from level 2: PLayer wins
		if(this.level.getTotNumItems() == player.score && level.getCode() ==2)
		{
			t1.stop(); // Stop the timers to stop updating locations 
			t2.stop();
			// Display Win message
			JOptionPane.showMessageDialog(null, "You won!!");
			System.exit(0);
		}
		//  If player has collected all the gems from level 1 advance to level 2
		else if(this.level.getTotNumItems() == player.score && level.getCode() ==1)
		{
			// Stop the timers
			t1.stop();
			t2.stop();
			// Get new monsters for level 2
			monsters.clear();
			monsters.add(new SimpleMonster(this,2,2,0,1));
			monsters.add(new Ghost(this,5,9,1,0));
			monsters.add(new Ghost(this,15,9,0,2));
			monsters.add(new Ghost(this,19,1,2,0));
			monsters.add(new Ghost(this,1,9,0,1));
			monsters.add(new SimpleMonster(this,9,15,1,1));
			monsters.add(new SimpleMonster(this,12,6,0,0));
			monsters.add(new SimpleMonster(this,5,18,2,0));
			// Reset player location
			player.row = 1;
			player.col = 1;
			// Message Display
			JOptionPane.showMessageDialog(null, "Proceed To level 2!!");
			// Reset Score
			this.player.score=0;
			// Instantiate new level
			this.level = new Level(this,this.rep2,2);
			// Repaint
			this.levelPanel.repaint();
			// Start the timers again
			t1.start();
			t2.start();
		}
	}

	/**
	 * Lose Method: Check to see if player is out of lives
	 */
	public void lose()
	{
		if(this.player.lives == 0)
		{
			t1.stop();
			JOptionPane.showMessageDialog(null, "Sorry you lost!!");
			System.exit(0);
		}
	}


	/**
	 * Constructor for class Game.
	 * Initializes all the instance variables
	 */
	public Game()
	{


		char o = 'o'; // Empty Floor
		char w = 'w'; // Wall
		char c = 'c'; // Coin or Gem
		char h = 'h'; // Health Pickup
		char t = 't'; // Teleport Tile


		// The map for level 1 (20 x 20 )
		rep1 = new char[][] {
				{c,c,c,c,c,c,c,c,c,c,c,c,c,w,w,w,w,w,c,w}   ,
				{c,o,w,c,w,c,c,c,w,c,c,c,c,c,c,c,c,c,c,w}	,
				{c,c,c,c,w,c,c,c,w,c,c,c,w,w,w,w,w,w,c,c}	,
				{c,t,c,c,w,c,c,c,w,c,c,c,c,c,c,c,c,c,c,c}	,
				{c,c,c,c,w,c,c,c,w,c,c,c,c,c,c,c,c,c,c,c}	,
				{c,c,c,c,w,c,c,c,w,c,c,w,w,w,w,w,w,w,w,c}	,
				{c,c,c,c,c,c,c,c,w,c,c,c,c,c,o,w,c,c,c,c}	,
				{c,c,c,c,c,c,c,c,w,c,c,w,w,w,w,w,c,c,c,c}	,
				{c,w,w,w,w,w,w,w,w,c,c,c,c,c,c,w,c,c,c,c}	,
				{c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c}	,
				{c,c,w,c,c,c,c,c,c,w,w,w,w,w,w,w,w,w,c,c}	,
				{c,c,w,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c}	,
				{c,c,w,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c}	,
				{c,c,w,c,c,c,c,w,w,w,w,w,w,w,w,w,w,c,c,c}	,
				{c,c,w,c,c,c,c,c,c,c,c,c,w,c,c,c,c,c,c,c}	,
				{c,c,w,c,c,w,w,w,w,w,w,w,w,w,w,w,w,w,c,c}	,
				{c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c}	,
				{c,c,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,c,c,c}	,
				{c,w,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,o,c}	,
				{w,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,h}	,
		};

		//The map for level 2 (20 x 20 )
		rep2 = new char[][] {{w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w}   ,
				{w,o,c,c,w,c,c,c,w,c,c,c,c,c,c,c,c,c,o,w}	,
				{w,c,c,c,w,c,c,c,w,c,c,c,w,w,w,w,w,w,c,w}	,
				{w,c,c,t,w,c,c,c,w,c,c,c,c,c,c,c,c,c,c,w}	,
				{w,c,c,c,w,c,c,c,c,c,c,c,c,c,c,c,c,c,c,w}	,
				{w,c,c,c,w,c,c,c,c,c,c,w,w,w,w,w,w,w,w,w}	,
				{w,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,w}	,
				{w,c,w,w,w,w,w,w,w,w,w,c,c,c,c,c,c,c,c,w}	,
				{w,c,c,c,c,w,c,c,c,c,c,c,c,w,w,w,w,w,c,w}	,
				{w,c,c,c,c,w,c,c,c,c,c,c,c,c,c,c,c,c,c,w}	,
				{w,c,w,c,c,w,c,c,c,w,w,w,w,w,w,w,w,w,c,w}	,
				{w,c,w,c,c,w,c,c,c,c,c,c,c,c,w,c,c,c,c,w}	,
				{w,c,w,c,c,w,c,c,c,c,c,c,c,c,w,c,c,c,c,w}	,
				{w,c,w,c,c,w,c,w,w,w,w,w,w,c,w,c,c,c,c,w}	,
				{w,c,w,c,c,c,c,c,c,c,c,c,c,c,w,c,c,c,c,w}	,
				{w,c,w,c,c,c,c,c,c,c,c,c,c,c,c,c,c,w,c,w}	,
				{w,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,w}	,
				{w,c,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,c,w}	,
				{w,o,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,c,h,w}	,
				{w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w,w}	,
		};



		// Instantiate Level 1
		level = new Level(this,rep1,1);

		player = new Player(this,1,1);

		monsters = new ArrayList<Monster>();
		monsters.add(new Ghost(this,5,9,1,0));
		monsters.add(new Ghost(this,15,9,0,2));
		monsters.add(new SimpleMonster(this,18,18,1,1));
		monsters.add(new SimpleMonster(this,10,12,2,0));
		monsters.add(new SimpleMonster(this,5,12,0,1));
		monsters.add(new SimpleMonster(this,14,6,0,0));
		// Display Score  and status
		score = new JLabel("Coins Collected : " + player.score + " / " +  level.getTotNumItems() + "      ||      ");
		status = new JLabel("Lives left: " + player.lives );
		// Instatiate LevelPanel
		levelPanel = new LevelPanel(this);

		// Instantiate and start the timers
		t1 = new Timer(750, new TimeListener(this));
		t2 = new Timer(1, new TimeListener2(this));
		t1.start();
		t2.start();

	}


	/**
	 * Main Program under Game class
	 * @param args Command line arguments
	 */
	public static void main (String[] args)
	{
		// Instantiate new Game object
		Game game = new Game();
		// Setup JFrame and add LevelPanel to displat
		JFrame frame = new JFrame ("Game");
		frame.setPreferredSize(new Dimension(430,470));
		frame.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
		frame.getContentPane().add(levelPanel);
		frame.pack();
		frame.setVisible(true);
	}

	/**
	 * Action Listener Class for timer1 slow
	 * @author Nakul Dureja
	 * @version 2.0 26 Nov 2012
	 *
	 */
	private class TimeListener implements ActionListener
	{
		//Game object is passed
		Game game;
		/**
		 * Parameterized Constructor for class timeListerner
		 * @param game Game object
		 */
		TimeListener(Game game)
		{
			this.game = game;
		}
		/**
		 * Action performed method by the the slow timer1 
		 * @param arg0 ActionEvent object
		 */
		public void actionPerformed(ActionEvent arg0) {


			// Update Monster Postions
			for(Monster m:game.monsters)
			{
				if(!m.equals(null))
					m.update();
			}
	

		}

	}

	/**
	 * Action Listener Class for timer2 fast
	 * @author Nakul Dureja
	 * @version 2.0 26 Nov 2012
	 */
	private class TimeListener2 implements ActionListener
	{

		Game game;
		/**
		 * Parameterized Constructor for class timeListerner
		 * @param game Game object
		 */
		TimeListener2(Game game)
		{
			this.game = game;
		}

		/**
		 * Action performed method by the the slow timer1
		 * @param arg0 ActionEvent object 
		 */
		public void actionPerformed(ActionEvent arg0) {

			// Refresh player stats
			game.setScore("Coins Collected : " + game.player.score + " / " +  game.level.getTotNumItems()  + "      ||      ");
			game.setStatus("Lives left: " + player.lives );
			// Detect Collision
			for(Monster m : game.monsters)
			{
				if(!m.equals(null))
					m.attackAction(game.player);

			}
			// Check for win/lose condition 
			game.lose();
			game.levelPanel.repaint(); // Repaint
			game.win();


		}

	}

	
	/**
	 * Getter for score
	 * @return score Jlabel of the player
	 */
	public JLabel getScore()
	{
		return score;
	}
	/**
	 * Setter for score
	 * @param s string to be set as JLabel
	 */
	public void setScore(String s)
	{
		this.score.setText(s);
	}
	
	/**
	 * Getter for status
	 * @return status JLabel of the player
	 */
	public JLabel getStatus()
	{
		return status;
	}
	/**
	 * Setter for status
	 * @param s string to be set as JLabel
	 */
	public void setStatus(String s)
	{
		this.status.setText(s);
	}

}
