import java.awt.*;
import javax.swing.ImageIcon;

/**
 * This class holds the 2d array of tiles and handles the input map.
 * @author Nakul Dureja
 * @version 2.0 
 *
 */
public class Level {

	// variables
	public static final int WIDTH  = 20;
	public static final int HEIGHT = 20;
	private static int code; // Level Code
	private Tile[][] plan = new Tile[WIDTH][HEIGHT];
	private char[][] tiles;
	private Game game;
	private int totNumItems,numItems;

	/**
	 * Constructor for the class
	 * @param game Game object
	 * @param levelRep level representation array char
	 * @param code level code (1 or 2)
	 */
	public Level(Game game, char[][] levelRep,int code)
	{
		this.code = code;
		this.setTiles(levelRep);
		this.game = game;

		// Initializing variables
		if(code == 1)
		{
			this.numItems = 290;
			this.totNumItems = 290;
		}
		else if(code==2)
		{
			this.numItems = 234;
			this.totNumItems = 234;	
		}
	}

	/**
	 * Draw Method to draw everything.
	 * @param g Graphics object
	 */
	public void draw(Graphics g)
	{

		/// Loop through the char array map and draw
		for(int i=0;i<getTiles().length;i++)
		{
			for(int j=0;j<getTiles()[i].length;j++)
			{

				//  Handles Floor
				if(this.getTiles()[i][j] == 'o')
				{	

					this.plan[i][j] = new Floor();
					this.plan[i][j].drawTile(g, j*WIDTH, i*HEIGHT);
				}
				//  Handles Wall
				else if(this.getTiles()[i][j] == 'w')
				{	

					this.plan[i][j] = new Wall();
					this.plan[i][j].drawTile(g, j*WIDTH, i*HEIGHT);
				}
				//  Handles Coins
				else if(this.getTiles()[i][j] == 'c')
				{	

					this.plan[i][j] = new Floor(new Gem());
					this.plan[i][j].drawTile(g, j*WIDTH, i*HEIGHT);
				}
				//  Handles Heath Pickups
				else if(this.getTiles()[i][j] == 'h')
				{	

					this.plan[i][j] = new Floor(new HealthPack());
					this.plan[i][j].drawTile(g, j*WIDTH, i*HEIGHT);
				}
				//  Handles Teleport Tiles
				else if(this.getTiles()[i][j] == 't')
				{	

					this.plan[i][j] = new TeleTile();
					this.plan[i][j].drawTile(g, j*WIDTH, i*HEIGHT);
				}

			}
		}

		// Draw the player
		game.player.draw(g);

		// Draw the monsters
		for(Monster m : game.monsters)
		{
			if(!m.equals(null))
				m.draw(g);
		}

	}



	/**
	 * Getter for numItems
	 * @return numItems Number of items left
	 */
	public int getNumItems()
	{
		return numItems;
	}



	/**
	 * Getter for totNumItems
	 * @return totNumItems Total Number of items left
	 */
	public int getTotNumItems()
	{
		return totNumItems;
	}
	
	/**
	 * Setter for numItems
	 * @param numItems number of items left
	 */
	public void setNumItems(int numItems)
	{
		this.numItems = numItems;
	}

	/**
	 * Getter for plan
	 * @return plan Tile[][]
	 */
	public Tile[][] getPlan()
	{
		return plan;
	}

	/**
	 * Getter for tiles
	 * @return tiles char[][]
	 */
	public char[][] getTiles() {
		return tiles;
	}

	/**
	 * Setter for tiles
	 * @param tiles char [][]
	 */
	public void setTiles(char[][] tiles) {
		this.tiles = tiles;
	}
	
	/**
	 * Getter for level Code
	 * @return code int
	 */
	public int getCode()
	{
		return code;
	}

}
