
import java.awt.*;
import javax.swing.ImageIcon;

/**
 * Extends Character. Represents the player. 
 * @author Nakul Dureja
 * @version 2.0 
 *
 */
public class Player extends Character{

	int score;
	int lives;
	// Image for the player
	static Image img = new ImageIcon(Player.class.getResource("/player.png")).getImage();
	/**
	 * Parameterized constructor for the class Player
	 * @param game Game object 
	 * @param row row value
	 * @param col col value
	 */
	public Player(Game game, int row, int col)
	{
		super(game, row, col);
		score = 0;
		lives = 3;
	}

	/**
	 * Move Method for the player
	 * @param drow move vertically
	 * @param dcol move horizontally
	 */
	public void move(int drow, int dcol)
	{

		// Check for bound conditions and move the player
		if( row+drow>=0 && col+dcol>=0 && row+drow<Level.WIDTH && col+dcol<Level.WIDTH && game.level.getPlan()[row+drow][col+dcol].isPassable() )
		{
			game.player.row = row+drow;
			game.player.col = col+dcol;

		}
		// Account for Teleport Tile
		if(game.level.getTiles()[row][col] == 't')
		{
			//  Level 1 Teleport
			if(game.level.getCode()== 1)
			{
				game.player.col= 13;
				game.player.row= 6;
				game.levelPanel.repaint();
			}
			// Level 2 Teleport
			else if(game.level.getCode()== 2)
			{
				game.player.col= 18;
				game.player.row= 2;
				game.levelPanel.repaint();
			}

		}
		// Check for coin
		if(game.level.getTiles()[row][col] == 'c')
		{
			//Pick coin and update score
			char[][] temp = game.level.getTiles();
			temp[row][col] = 'o';
			game.level.setTiles(temp);
			score++;
			game.level.setNumItems( (game.level.getNumItems()) -1   );

		}
		// Check for health pickup
		else if(game.level.getTiles()[row][col] == 'h')
		{
			//Pick health boost and update lives
			char[][] temp = game.level.getTiles();
			temp[row][col] = 'o';
			game.level.setTiles(temp);
			lives++;


		}

	}

	/**
	 * Overridden draw method.
	 * Draws the player
	 * @param g Graphics Object
	 */
	public void draw(Graphics g) {
		g.drawImage(this.img,col*Level.WIDTH, row*Level.HEIGHT,Level.WIDTH,Level.HEIGHT,null);
	}

	
	//Unused methods
	/**
	 * Overridden unused update method.
	 * Updates the player  position.
	 */
	public void update() {}





}
